'' =================================================================================================
''
''   File....... jm_lanc_commander.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 18 MAY 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28

  SIO = 26                                                      ' lanc serial io
  

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


obj

  lanc : "jm_lanc"
  term : "fullduplexserial64"
 

var

  long  tc

  byte  buffer[8]
  byte  mode
  byte  recording

  byte  guide


dat

  Screen                byte    "LANC Monitor/Controller", CR
                        byte    CR
                        byte    "-- -- -- -- -- -- -- --", CR
                        byte    CR
                        byte    "Online..... No", CR
                        byte    "Mode....... ", CR
                        byte    "Timecode... ", CR
                        byte    CR
                        byte    0

  Sign                  byte    "+"
                     

pub main | f1, f2, idx, c

  input(SIO)                                                    ' clear in this cog
  lanc.start(SIO)                                               ' lanc rx
  term.start(RX1, TX1, %0000, 115_200)                          ' terminal
  pause(5)

  term.tx(CLS)
  term.str(@Screen)

  f1 := lanc.frame                                              ' get frame counter
  
  repeat
    c := ucase(term.rxcheck)                                    ' check keyboard
    if (c => 0)                                                 ' if key...
      case c
        "P": { power }
          if (lanc.available)                                   ' lanc signal present?
            lanc.command($18, $5E)                              ' power down
          else
            lanc.wake_up                                        ' attempt wake-up
          term.rxflush
          
        "R": { toggle record }
          lanc.command(18, $33)
          term.rxflush
          
        "A": { toggle auto focus }
          lanc.command($28, $41)
          term.rxflush    

        "N": { focus near }
          lanc.command($28, $47) 

        "F": { focus far }
          lanc.command($28, $45)

        "T": { zoom tele - fast }
          lanc.command($28, $0E) 
           
        "W": { zoom wide - fast }   
          lanc.command($28, $1E)


    f2 := lanc.frame                                            ' get frame counter  

    if (f2 == 0)
      no_lanc

    elseif (f2 <> f1)                                           ' new frame?
      bytemove(@buffer, lanc.address, 8)                        ' capture to local buffer  
      move_to(0, 2)                                             ' display frame     
      repeat idx from 0 to 7                                                          
        term.hex(buffer[idx], 2)                                                      
        term.tx(" ")                                                                  

      mode := buffer[4] & %0000_0010                            ' get camcorder mode                
      has_lanc                                                  ' show status             

      f1 := f2                                                  ' save for next check


pub has_lanc | idx

  move_to(12, 4)
  term.str(string("Yes"))                                       ' show connected

  move_to(12, 5)                                                ' display camcorder mode
  if (mode)
    term.str(string("VTR"))
  else
    term.str(string("CAM"))

  move_to(16, 5)                                                ' display device status
  if (mode)
    case buffer[4]
      $02:
        term.str(string("STOP", CLREOL))
        
      $03:
        term.str(string("FWD", CLREOL))
                 
      $06:
        term.str(string("PLAY", CLREOL))
            
      $07:
        term.str(string("PAUSE", CLREOL))
        
      $46:
        term.str(string("PLAY >>", CLREOL))
        
      $56:
        term.str(string("PLAY <<", CLREOL))
        
      $83:
        term.str(string("RWD", CLREOL))
        
      other:
        term.tx(CLREOL)

  else
    case buffer[4]  
      $04:
        term.str(string("RECORD", CLREOL))
        
      $14:
        term.str(string("PAUSE", CLREOL))
        
      other:
        term.tx(CLREOL)

  guide := buffer[5] & $F0                                      ' guide for bytes 6/7
                     
  if (guide == $30)
    tc.byte[2] := buffer[7]                                     ' minutes
    tc.byte[1] := buffer[6]                                     ' seconds

  if (guide == $40)
    tc.byte[3] := buffer[6]                                     ' hours
    tc.byte[0] := buffer[7] & $3F                               ' frames
    if (buffer[7] & $80)
      Sign := "-"
    else
      Sign := "+"
    
  move_to(12, 6)                                                ' display timecode
  if (tc.byte[3] <> $FF)
    term.tx(sign)
    repeat idx from 3 to 0
      term.hex(tc.byte[idx], 2)
      if (idx > 0)
        term.tx(":")

  else
    term.tx(CLREOL)   

    
pub no_lanc

  move_to(0, 2)
  term.str(string("-- -- -- -- -- -- -- --"))

  move_to(12, 4)
  term.str(string("No "))    

  move_to(12, 5)
  term.tx(CLREOL)

  move_to(12, 6)
  term.tx(CLREOL)


pub move_to(x, y)

'' Move cursor for PST

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y)


pub ucase(c)

'' Converts lowercase alpha to uppercase

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  